@@ -24,6 +24,16 @@ sub new {
 	$self;
 }
 
+sub message {
+	return shift->SUPER::message() if @_ == 1;
+	my ($self,$msg) = @_;
+	if( $msg =~ /^internal error|Input is not proper UTF-8/ && $self->record ) {
+		my @ids = map { $_->identifier } $self->record;
+		$msg = "Parser error near record " . @ids . " [" . (pop(@ids) || pop(@ids) || 'identifier unknown') . "]: " . $msg;
+
+	}
+	return $self->SUPER::message($msg);
+}
 sub resumptionToken { shift->headers->header('resumptionToken',@_) }
 
 sub record {
@@ -38,7 +48,7 @@ sub next {
 	my $self = shift;
 	my $value = shift @{$self->{record}};
 	return $value if $value;
-	return undef if (!$self->{'resume'} || !$self->resumptionToken || $self->resumptionToken->is_empty);
+	return undef if (!$self->harvestAgent->resume || !$self->resumptionToken || $self->resumptionToken->is_empty);
 
 	my $r = $self->resume(resumptionToken=>$self->resumptionToken);
 	return $r->is_success ? $self->next : $r;
